@@ -1,3 +1,4 @@
 utils::globalVariables(c("x", "..density..", "variable", "value", "xbar",
                          "c_i_low", "c_i_high", "dens_fill", "dens_xbar",
-                         "dens_low", "dens_high", "year", "y"))
+                         "dens_low", "dens_high", "year", "y", "start",
+                         "end", "y.lab", "node", "result"))
